const os = require('os');
const WebSocket = require('ws');

// 端口
const port = 10086;

// 设备标识
const deviceFlag = "necAnalyzer";
const clientFlag = "necAnalyzerClient";

// 获取内网IP地址
const networkInterfaces = os.networkInterfaces();
var internalIP = null;
for (const name of Object.keys(networkInterfaces)) {
    for (const netIF of networkInterfaces[name]) {
        if (netIF.family === 'IPv4' && !netIF.internal) {
            internalIP = netIF.address;
            break;
        }
    }
    if (internalIP) break;
}

if (internalIP === null) {
    const readline = require('readline').createInterface({
        input: process.stdin,
        output: process.stdout
    });

    return readline.question(`获取IP地址失败，请检查网络连接`, function () {
        readline.close();
    });
}

// 创建WebSocket服务端实例，指定服务端监听的端口和地址
const ws = new WebSocket.Server({ port: port });

ws.on('listening', () => {
    console.log('websocket服务已启动');
    console.log("IP:", internalIP, ",端口:", port);
});

// 存储客户端连接的映射
const clients = new Map();

// 监听连接事件
ws.on('connection', function connection(ws, req) {

    // 从请求头中获取客户端标识
    const clientId = req.headers['sec-websocket-protocol'];

    if (clientId !== deviceFlag && clientId !== clientFlag) {
        let errMsg = clientId === undefined ? '无连接标识': `标识: ${clientId} 异常`;
        console.log(`发现新连接，${errMsg} ，已主动关闭该连接`);
        return ws.close(1002, `标识: ${clientId} 异常`);
    }

    // 发送连接标识
    if (clientId === clientFlag) {
        ws.send(676);
    }

    console.log((clientId === clientFlag ? '客户端' : '采集终仪') + '已接入')

    // // 将客户端连接存储在映射中
    clients.set(clientId, ws);

    // 监听消息事件
    ws.on('message', function incoming(message) {
        // 根据客户端标识转发消息
        forwardMessage(clientId, message);
    });

    // 监听关闭连接事件
    ws.on('close', function close(error, message) {
        console.log((clientId === clientFlag ? '客户端' : '采集终仪') + "断开连接");
        if (clients.has(clientId)) {
            clients.delete(clientId);
        }
    });

});

// 监听错误事件
ws.on('error', function error(error) {
    if (error.code === 'EADDRINUSE') {
        console.error(`端口 ${port} 已被占用，请更换端口号。`);
    } else {
        console.error('发生未知错误：', error);
    }
});

function forwardMessage(clientId, message) {

    if (clients.has(client)) {
        const targetClient = clients.get(clientId);
        if (targetClient.readyState === WebSocket.OPEN) {
            targetClient.send(`${message}`);
        } else {
            clients.delete(clientId);
        }
    }
}